/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.define('DSS.addon.SceneResponder', {

    extend: 'DSS.addon.Framework',

    config: {
        appName: 'SceneResponder',
        appId: 'system-addon-scene-responder',
        appVersion: '1.0.3',
        appIcon: 'images/app_icon.png',
    },

    getHelp: function() {
        return Ext.create('Ext.Component', {
            loader: {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad: true,
                disableCaching: false
            }
        });
    },

    getContent: function() {
        var me=this;
        /* Parameter:
         *   * editor config
         *   * button config
         *   * store config   
         *   * columns
         *   * defaultSortCol
         * */
        
        var oConfig={
                defaultSortCol:'name',
                disabledProperty:'disabled',
                storeConfig: {
                    storeId : 'sceneresponderStore',
                    fields: [
                        { name: 'id' },
                        { name: 'disabled',  sortType: 'asBoolean' },
                        { name: 'name',     sortType: 'asUCText'  },
                        { name: 'actions',  sortType: 'asAction'  },
                        { name: 'triggers', sortType: 'asTrigger' },
                        { name: 'delay',    sortType: 'asTimespan' }
                    ]
                },
                columns: [{
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                },  {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width : 200
                }, {
                    header: _('Trigger'),
                    dataIndex: 'triggers', // alibi index
                    xtype: 'triggercolumn',
                    maintainFlex: true,
                    flex: 2,
                }, {
                    header : _('Activity'),
                    dataIndex: 'actions', // alibi index
                    xtype: 'actioncolumn',
                    maintainFlex : true,
                    flex : 5,
                }, {
                    header: _('Delay'),
                    dataIndex: 'delay',
                    xtype: 'timespancolumn',
                    width: 80
                }],     
                buttonText: {
                    newButton: _('New automaton'),
                    editButton: _('Edit automaton'),
                    enableButton:_('Enable automaton'),
                    disableButton:_('Disable automaton'),
                    testButton: _('Test automaton'),
                    deleteButton: _('Delete automaton'),
                },
                editorConfig:{
                    title : _('Edit automaton'),
                    staticControls : [ {
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type: 'hidden',
                        label: 'id',
                        id: 'id'
                    }],
                    tabControls : [{
                        tabLabel : _('Trigger'),
                        tabDescription : _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
                        content : [{
                            type : 'TriggerConfigurator',
                            id : 'triggers',
                            flex : true
                       }, {
                            type: 'timespan',
                            label: _('Delay'),
                            id: 'delay'
                        }]
                    }, {
                        tabLabel : _('Activity'),
                        tabDescription : _('Choose the activities, which should be executed.'),
                        content : [{
                            type : 'ActionConfigurator',
                            id : 'actions',
                            flex : true
                        }]
                    }]
                },
                listeners: {
                    'loadDataStore':function (grid){
                        me.loadParameter(grid);
                    },
                    'saveEntry':function (grid,values){                        
                        dss.raiseEvent('system-addon-scene-responder.config', {
                            actions : 'save',
                            value : Ext.JSON.encode(values)
                        })
                        me.waitAndReload(grid);
                    },
                    'enableEntry':function (grid,entry){
                        dss.raiseEvent('system-addon-scene-responder.config', {
                            actions : 'enable',
                            id : entry.get('id')
                        });
                        me.waitAndReload(grid);
                     },
                    'disableEntry':function (grid,entry){
                        dss.raiseEvent('system-addon-scene-responder.config', {
                            actions : 'disable',
                            id : entry.get('id')
                        })
                        me.waitAndReload(grid);
                    },
                    'deleteEntry':function (grid,entry){
                        dss.raiseEvent('system-addon-scene-responder.config', {
                            actions : 'delete',
                            id : entry.get('id')
                        })
                        me.waitAndReload(grid);
                    },
                    'testEntry':function (grid,entry){
                        dss.ajaxAssyncRequestWithoutAnswer("/json/event/raise",
                                {name:'action_execute', parameter:  "path=/scripts/system-addon-scene-responder/entries/" + entry.get('id')});                       
                    }
                }
        };
        
        return Ext.create('DSS.gridPanel', oConfig);
    },
    waitAndReload : function(grid) {
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'system-addon-scene-responder.saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter(grid);
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'system-addon-scene-responder.saved'
            });
        });
    },
    loadParameter : function(grid) {
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/system-addon-scene-responder/entries/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        var oResultObject = dss.parsePropertyQueryAnswer(result, [
            {name: 'name', type: 'simple'},
            {name: 'id', type: 'simple'},
            {name: 'disabled', type: 'condition/enabled'},
            {name: 'delay', type: 'simple'},
            {name: 'actions', type: 'action'},
            {name: 'triggers', type: 'trigger'}
        ]);
        
        grid.loadParameter(oResultObject);
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/scene-responder.po']);
    var sceneresponder = Ext.create('DSS.addon.SceneResponder');
    sceneresponder.initPage();
});

